@@ -1,5 +1,7 @@
 import logging
 import os
+import tarfile
+import zipfile
 
 import bbcloud_python_sdk as utils
 from .ObsFileCache import ObsFileCache
@@ -138,36 +140,72 @@ class FileCache():
 
     @runtime_logging
     def get(self, key, local_file, default=None, auto_unzip=False, auto_delete_local_zip=True, auto_untar=False,
-            auto_delete_local_tar=True):
+            auto_delete_local_tar=True, retry=1):
+        """
+        :param key: 文件缓存key
+        :param local_file: 本地保存路径
+        :param default: 如果没有获取到文件缓存，则使用默认替代资源地址，替代资源如果获取成功会被存到文件缓存系统
+        :param auto_unzip:如果是zip类型文件缓存，是否自动解压缩
+        :param auto_delete_local_zip:zip自动解压缩后是否删除压缩包
+        :param auto_untar:如果是tar类型文件缓存，是否自动解压缩
+        :param auto_delete_local_tar:tar自动解压缩后是否删除压缩包
+        :param retry:重试次数，默认为1，获取数据异常时使用，一般不修改
+        :return:
+        """
+        if retry >= 0:
+            if not os.path.exists(os.path.dirname(local_file)):
+                utils.make_dir(os.path.dirname(local_file))
 
-        if not os.path.exists(os.path.dirname(local_file)):
-            utils.make_dir(os.path.dirname(local_file))
+            is_get = self.default_engine.set_namespace(namespace=self.namespace).get(key=key, local_file=local_file)
+            if not is_get:
+                if default:
+                    self.set(key=key, file_path=default, del_local=True)
+                    is_get = self.get(key=key, local_file=local_file)
+                else:
+                    return False
 
-        is_get = self.default_engine.set_namespace(namespace=self.namespace).get(key=key, local_file=local_file)
-        if not is_get:
-            if default:
-                self.set(key=key, file_path=default, del_local=True)
-                is_get = self.get(key=key, local_file=local_file)
+            if is_get:
+                if auto_unzip or auto_untar:
+                    is_unpack = self.unpack(local_file=local_file)
+                    if not is_unpack:
+                        os.remove(local_file)
+                        return self.get(
+                            key=key,
+                            local_file=local_file,
+                            default=default,
+                            auto_unzip=auto_unzip,
+                            auto_delete_local_zip=auto_delete_local_zip,
+                            auto_untar=auto_untar,
+                            auto_delete_local_tar=auto_delete_local_tar,
+                            retry=retry - 1
+                        )
+                    if auto_delete_local_zip or auto_delete_local_tar:
+                        os.remove(local_file)
+                return True
             else:
                 return False
-
-        if is_get:
-            if os.path.splitext(local_file)[-1] == '.zip' and auto_unzip:
-                unzip_dir = os.path.dirname(local_file)
-                if os.path.exists(local_file):
-                    utils.unzip(file_name=local_file, dst_dir=unzip_dir)
-                    if auto_delete_local_zip:
-                        os.remove(local_file)
-            elif os.path.splitext(local_file)[-1] == '.gz' and auto_untar:
-                untar_dir = os.path.dirname(local_file)
-                if os.path.exists(local_file):
-                    utils.untar(file_name=local_file, dst_dir=untar_dir)
-                    if auto_delete_local_tar:
-                        os.remove(local_file)
-            return True
         else:
             return False
 
+    def unpack(self, local_file):
+        if os.path.splitext(local_file)[-1] == '.zip':
+            unzip_dir = os.path.dirname(local_file)
+            if os.path.exists(local_file):
+                if zipfile.is_zipfile(local_file):
+                    utils.unzip(file_name=local_file, dst_dir=unzip_dir)
+                    return True
+
+        elif os.path.splitext(local_file)[-1] == '.gz':
+            untar_dir = os.path.dirname(local_file)
+            if os.path.exists(local_file):
+                if tarfile.is_tarfile(local_file):
+                    utils.untar(file_name=local_file, dst_dir=untar_dir)
+                    return True
+
+        logging.info('解压失败，请检查压缩文件格式：%s' % local_file)
+
+        return False
+
     @runtime_logging
     def delete(self, key=None, delete_all_area=False):
         if delete_all_area:
@@ -191,3 +229,7 @@ class FileCache():
     @runtime_logging
     def list_dir(self, deep_num):
         return self.default_engine.set_namespace(namespace=self.namespace).list_dir(deep_num=deep_num)
+
+    @runtime_logging
+    def copy(self, target_namespace, key=None):
+        return self.default_engine.set_namespace(namespace=self.namespace).copy(target_namespace, key)
